# -*- coding: utf-8 -*-
"plugin for value: set it in sqlalchemy"
# Copyright (C) 2013-2014 Team tiramisu (see AUTHORS for all contributors)
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ____________________________________________________________


from ..util import Cache
from .util import SqlAlchemyBase
import util
from sqlalchemy import Column, Integer, String, PickleType
from tiramisu.setting import owners


#____________________________________________________________
#
# information
class _Vinformation(SqlAlchemyBase):
    __tablename__ = 'vinformation'
    id = Column(Integer, primary_key=True)
    session = Column(String, index=True)
    path = Column(String, index=True)
    key = Column(String)
    value = Column(PickleType)

    def __init__(self, session, key, value):
        self.session = session
        self.key = key
        self.value = value


class Value(SqlAlchemyBase):
    __tablename__ = 'value'
    id = Column(Integer, primary_key=True)
    session = Column(String, index=True)
    path = Column(String, index=True)
    key = Column(String)
    value = Column(PickleType)
    owner = Column(String)

    def __init__(self, session, path, value, owner):
        self.session = session
        self.path = path
        self.value = value
        self.owner = owner


class Values(Cache):

    # value
    def setvalue(self, path, value, owner):
        """set value for a path
        a specified value must be associated to an owner
        """
        val = util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first()
        if val is None:
            util.session.add(Value(self._storage.session_id, path, value,
                                   owner))
        else:
            val.value = value
            val.owner = owner
        util.session.commit()

    def getvalue(self, path):
        """get value for a path
        return: only value, not the owner
        """
        val = util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first()
        if not val:
            raise KeyError('no value found')
        return val.value

    def hasvalue(self, path):
        """if path has a value
        return: boolean
        """
        return util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first() is not None

    def resetvalue(self, path):
        """remove value means delete value in storage
        """
        val = util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first()
        if val is not None:
            util.session.delete(val)
            util.session.commit()

    def get_modified_values(self):
        """return all values in a dictionary
        example: {'path1': (owner, 'value1'), 'path2': (owner, 'value2')}
        """
        ret = {}
        for val in util.session.query(Value).filter_by(
                session=self._storage.session_id).all():
            ret[val.path] = (val.owner, val.value)
        return ret

    # owner
    def setowner(self, path, owner):
        """change owner for a path
        """
        val = util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first()
        if val is None:
            raise KeyError('no value found')
        else:
            val.owner = owner
        util.session.commit()

    def getowner(self, path, default):
        """get owner for a path
        return: owner object
        """
        val = util.session.query(Value).filter_by(
            path=path, session=self._storage.session_id).first()
        if val is None:
            return default
        else:
            owner = val.owner
            # autocreate owners
            try:
                return getattr(owners, owner)
            except AttributeError:
                owners.addowner(owner)
                return getattr(owners, owner)

    def set_information(self, key, value):
        """updates the information's attribute
        (which is a dictionary)

        :param key: information's key (ex: "help", "doc"
        :param value: information's value (ex: "the help string")
        """
        pass
        val = util.session.query(_Vinformation).filter_by(
            key=key, session=self._storage.session_id).first()
        if val is None:
            util.session.add(_Vinformation(self._storage.session_id, key,
                                           value))
        else:
            val.value = value
        util.session.commit()

    def get_information(self, key):
        """retrieves one information's item

        :param key: the item string (ex: "help")
        """
        val = util.session.query(_Vinformation).filter_by(
            key=key, session=self._storage.session_id).first()
        if not val:
            raise ValueError("not found")
        return val.value
